@@ -10,7 +10,7 @@ __all__ = """
 """.split()
 
 from pyoverload import *
-from .tensor import Tensor, ones
+from .tensor import Tensor, Size, ones
 from pyctlib import restore_type_wrapper
 from pyctlib import vector
 import torch
@@ -22,19 +22,33 @@ def crop_as(x: Array, y: tuple, center: tuple, fill: Scalar=0) -> Array:
     x = Tensor(x)
     size_x = x.shape
     size_y = y
-    if len(size_y) < len(size_x) - 1 or len(size_y) < len(size_x) and not x.batch_dimension:
-        raise TypeError("Need more dimensions in size y, please use -1 if the dimension doesn't need to be cropped. ")
-    if len(size_y) > len(size_x): raise TypeError("Too many dimensions in target size y, please check your input.")
-    if len(size_y) == len(size_x) - 1: size_y = size_y[:x.batch_dimension] + (-1,) + size_y[x.batch_dimension:]
-    assert len(size_x) == len(size_y)
+
+    if isinstance(size_y, Size) and size_x.nspace == size_y.nspace:
+        size_y = tuple(size_y.space)
+    size_y = tuple(size_y)
+    if len(size_y) == len(size_x): pass
+    elif len(size_y) == size_x.nspace:
+        s = size_x.special
+        if len(s) == 0: pass
+        elif len(s) == 1: size_y = size_y[:s[0]] + (-1,) + size_y[s[0]:]
+        else: size_y = size_y[:s[0]] + (-1,) + size_y[s[0]:s[1]] + (-1,) + size_y[s[1]:]
+    else: raise TypeError("Mismatch dimensions in 'crop_as', please use -1 if the dimension doesn't need to be cropped. ")
+    assert len(size_y) == len(size_x)
     size_y = tuple(a if b == -1 else b for a, b in zip(size_x, size_y))
-    if len(center) < len(size_x) - 1 or len(center) < len(size_x) and not x.batch_dimension:
-        raise TypeError("Need more dimensions in center, please use -1 if the dimension that is centered or doesn't need cropping. ")
-    if len(center) > len(size_x): raise TypeError("Too many dimensions in center, please check your input.")
-    if len(center) == len(size_x) - 1: center = center[:x.batch_dimension] + (-1,) + center[x.batch_dimension:]
+
+    if len(center) == len(size_x): pass
+    elif len(center) == size_x.nspace:
+        s = size_x.special
+        if len(s) == 0: pass
+        elif len(s) == 1: center = center[:s[0]] + (-1,) + center[s[0]+1:]
+        else: center = center[:s[0]] + (-1,) + center[s[0]+1:s[1]] + (-1,) + center[s[1]+1:]
+    elif len(x for x in center if x >= 0) == len(x for x in size_y if x >= 0):
+        center = tuple(a if b >= 0 else -1 for a, b in zip(center, size_y))
+    else: raise TypeError("Mismatch dimensions for the center in 'crop_as', please use -1 if the dimension that is centered or doesn't need cropping. ")
     assert len(center) == len(size_x)
     center = tuple(a / 2 if b == -1 else b for a, b in zip(size_x, center))
-    z = Tensor(fill) * ones(*size_y).type_as(x)
+
+    z = fill * ones(*size_y).type_as(x)
     def intersect(u, v):
         return max(u[0], v[0]), min(u[1], v[1])
     z_box = [intersect((0, ly), (- round(float(m - float(ly) / 2)), - round(float(m - float(ly) / 2)) + lx)) for m, lx, ly in zip(center, size_x, size_y)]
@@ -45,11 +59,12 @@ def crop_as(x: Array, y: tuple, center: tuple, fill: Scalar=0) -> Array:
     region_x = tuple(slice(u, v) for u, v in x_box)
     z[region_z] = x[region_x]
     z.roi = region_x
+    z.special_from_(x)
     return z
 
 @overload
 def crop_as(x: Array, y: Array, center: tuple, fill: Scalar=0) -> Array:
-    return crop_as(x, tuple(y.shape), center, fill)
+    return crop_as(x, y.shape, center, fill)
 
 @overload
 def crop_as(x: Array, y: [tuple, Array], fill: Scalar=0) -> Array:
